import json
import base64

def serialize(value):
    """Base64 encode an JSON-serializable object."""
    return base64.b64encode(json.dumps(value).encode("utf-8")).decode("utf-8")


def deserialize(value):
    """Decode a Bse64 string into a JSON object."""
    return json.loads(base64.b64decode(value).decode("utf-8"))



import __main__
import os
import re
import sys
import json
import errno
import random
import string
import urllib
import shutil
import logging
from typing import Dict, Any

log = logging.getLogger(__name__)

def get_main_source_path():
    """Get the absolute path to the program entrypoint source."""
    return os.path.abspath(__main__.__file__)

def main_source_lives_in_cwd():
    """Check if the main executable is located at the root of CWD."""
    return os.path.dirname(get_main_source_path()) == os.getcwd()

def random_string(size=5, chars=string.ascii_lowercase + string.digits):
    """Generate random string."""
    return "".join(random.choice(chars) for _ in range(size))

def abs_working_dir(path):
    """Get absolute path to parent dir."""
    abs_path = os.path.abspath(path)
    if os.path.isfile(path):
        abs_path = os.path.dirname(abs_path)
    return abs_path

def rm_r(path, ignore_missing=True, silent=False):
    """Remove a file or directory.

    Similar to rm -r. If the path does not exist and ignore_missing is False,
    OSError is raised, otherwise it is ignored.
    If silent is True, nothing is raised.
    """
    def onerror(function, path, excinfo):
        # Function to handle ENOENT in shutil.rmtree()
        e = excinfo[1]
        if (ignore_missing and isinstance(e, OSError)
                and e.errno == errno.ENOENT):
            return
        raise e

    log.info("Removing path `%s'", path)

    try:
        if os.path.isfile(path) or os.path.islink(path):
            os.remove(path)
        elif os.path.isdir(path):
            shutil.rmtree(path, onerror=onerror)
        elif os.path.exists(path) and not silent:
            raise RuntimeError("Failed to remove path `%s': Path exists but is"
                               " not a file nor a directory" % path)
        else:
            # The path does not exists, raise the appropriate exception and let
            # the exception handler handle it (i.e., check ignore_missing etc.)
            raise OSError(errno.ENOENT, "No such file or directory", path)
    except OSError as e:
        if silent:
            log.debug("Path `%s' does not exist, skipping removing it", path)
            return
        if (not ignore_missing) or (e.errno != errno.ENOENT):
            log.error("Failed to remove path `%s' (errno: %s): %s",
                      path, e.errno, e)
            raise

def remove_ansi_color_sequences(text):
    """Remove ANSI color sequences from text."""
    ansi_color_escape = re.compile(r'\x1B\[[0-9;]*m')
    return ansi_color_escape.sub('', text)

def comment_magic_commands(code):
    """Comment the magic commands in a code block."""
    magic_pattern = re.compile(r'^(\s*%%?.*)$', re.MULTILINE)
    return re.sub(magic_pattern, r'#\1', code.strip())

def encode_url_component(component: str):
    """Encode a value so it can safely be used as a URL component."""
    return urllib.parse.quote(component, safe="")

def sanitize_k8s_name(name):
    """Sanitize a string to conform to Kubernetes naming conventions."""
    name = re.sub("-+", "-", re.sub("[^-0-9a-z]+", "-", name.lower()))
    return name.lstrip("-").rstrip("-")

def is_ipython() -> bool:
    """Returns whether the code is running in a ipython kernel."""
    try:
        import IPython
        ipy = IPython.get_ipython()
        if ipy is None:
            return False
    except ImportError:
        return False
    return True

def read_json_from_file(path: str) -> Dict:
    """Read a file that contains a JSON object and return it as dictionary."""
    try:
        return json.loads(open(path, 'r').read())
    except json.JSONDecodeError:
        log.exception("Failed to parse json file %s", path)
        raise

def ensure_or_create_dir(filepath: str):
    """Ensure the dir of a file exists and isdir, or create it.

    Raises:
        RuntimeError: if dirname of filepath exists and is not a directory
    """
    dirname = os.path.dirname(filepath)
    if not os.path.exists(dirname):
        os.makedirs(dirname)
    elif not os.path.isdir(dirname):
        raise RuntimeError("'%s' is not a directory" % dirname)

def clean_dir(path: str):
    """If path exists, remove and then create empty dir."""
    if os.path.exists(path):
        shutil.rmtree(path)
    os.makedirs(path)

def shorten_long_string(obj: Any, chars: int = 75):
    """Shorten the string representation of the input object."""
    str_input = str(obj)
    return str_input[:chars] + " ..... " + str_input[len(str_input) - chars:]

def dedent(text: str):
    """Remove longest common prefix consisting of whitespaces.

    Args:
        text: Multiline string
    """
    matches = re.findall(r"(?m)^\s+", text)
    if len(matches) < len(text.splitlines()):
        return text
    return re.sub(r"(?m)^.{%d}" % min(map(len, matches)), "", text)



import sys
import logging
import importlib
from modeldeploy_proxy_controller.common.utils import random_string
from modeldeploy_proxy_controller.rpc import errors, utils
from modeldeploy_proxy_controller.rpc.log import create_adapter, setup_logging

logger = create_adapter(logging.getLogger(__name__))

def import_func(request, import_func_str):
    """Import and return a function from a string."""
    mod_str, _sep, func_str = import_func_str.rpartition('.')
    if mod_str:
        mod_str = "modeldeploy_proxy_controller.rpc." + mod_str
    else:
        mod_str = "modeldeploy_proxy_controller.rpc"
    try:
        mod = importlib.import_module(mod_str)
        return getattr(mod, func_str)
    except (ValueError, AttributeError) as e:
        request.log.exception("Exception while importing function '%s' of" " module '%s': %s", func_str, mod_str, e)
        raise ImportError("Function `%s' of module `%s' cannot be found: %s" % (func_str, mod_str, e))

def format_success(result, trans_id):
    """Serialise the result."""
    return utils.serialize({"code": errors.Code.OK.value, "result": result, "trans_id": trans_id})

class ModelDeploymentControllerRPCRequest(object):
    """RPC request."""

    def __init__(self, trans_id=None, nb_path=None):
        if not trans_id:
            trans_id = random_string(size=10)
        self.log = create_adapter(logging.getLogger(__name__), trans_id, nb_path)
        self.trans_id = trans_id
        self.nb_path = nb_path

def sanitize_ctx(request, ctx):
    """Keep just the known context fields."""
    nb_path = ctx.pop("nb_path", None)
    if ctx:
        request.log.debug("Ignoring context fields: %s", ", ".join(ctx.keys()))
    return {"nb_path": nb_path}

def run(func, kwargs, ctx):
    """Execute command requests coming from the UI.

    Args:
        func: the name of function to be run
        kwargs: A dict object with the arguments to be passed to the function
        ctx: context

    Returns: The result of the called function
    """
    # Setup initial request obj to have something to log to
    request = ModelDeploymentControllerRPCRequest()
    setup_logging(request)
    request.log.debug("Decoding ctx of RPC function '%s'", func)
    try:
        ctx = utils.deserialize(ctx)
    except Exception:
        exc_info = sys.exc_info()
        request.log.exception("Failed to decode ctx: %s", ctx)
        return errors.RPCEncodingError(message=str(exc_info[1]),
                                       trans_id=request.trans_id).serialize()
    # Sanitize ctx and renew the request obj
    ctx = sanitize_ctx(request, ctx)
    request = ModelDeploymentControllerRPCRequest(request.trans_id, **ctx)

    request.log.debug("Decoding kwargs of RPC function '%s'", func)
    try:
        kwargs = utils.deserialize(kwargs)
        request.log.debug(kwargs)
    except Exception:
        exc_info = sys.exc_info()
        request.log.exception("Failed to decode kwargs: %s", kwargs)
        return errors.RPCEncodingError(message=str(exc_info[1]),
                                       trans_id=request.trans_id).serialize()

    request.log.debug("Importing RPC function '%s'", func)
    try:
        func = import_func(request, func)
    except ImportError as e:
        exc_info = sys.exc_info()
        request.log.exception("Failed to import RPC function '%s'", func)
        return errors.RPCImportError(message=str(e), trans_id=request.trans_id).serialize()

    request.log.info("Executing RPC function '%s(%s)'", func.__name__, ", ".join("%s=%s" % i for i in kwargs.items()))
    try:
        result = func(request, **kwargs)
        return format_success(result, request.trans_id)
    except errors._RPCError as e:
        request.log.exception("RPC function '%s' raised an RPCError",
                              func.__name__)
        return e.serialize()
    except Exception:
        exc_info = sys.exc_info()
        request.log.exception(("RPC function '%s' raised an unhandled"
                               " exception"), func.__name__)
        return errors.RPCUnhandledError(message=str(exc_info[1]), trans_id=request.trans_id).serialize()



import os
import requests
from modeldeploy_proxy_controller.rpc.nb import parse_notebook
import importlib.resources as resources
from modeldeploy_proxy_controller import res

API_HOST_URL = os.environ.get("API_HOST_URL", "")
API_VERSION = os.environ.get("API_VERSION", "v1")
PROXY_API_PREFIX = "{}/api/{}/".format(API_HOST_URL, API_VERSION)
TRANSFORMER_UPLOAD_URL = "{}/transformer/upload".format(PROXY_API_PREFIX)
REQUIREMENTS_UPLOAD_URL = "{}/requirements/upload".format(PROXY_API_PREFIX)
STATUS_URL = "{}/proxy/health/status".format(PROXY_API_PREFIX)

def revise_proxy_api_url(request, proxy_url):
    request.log.debug("revise_proxy_api_url")

    if not proxy_url:
        raise RuntimeError("Proxy URL should not be empty!")

    global PROXY_API_PREFIX
    global TRANSFORMER_UPLOAD_URL
    global REQUIREMENTS_UPLOAD_URL
    global STATUS_URL

    PROXY_API_PREFIX = "{}/api/{}/".format(proxy_url, API_VERSION)
    TRANSFORMER_UPLOAD_URL = "{}/transformer/upload".format(PROXY_API_PREFIX)
    REQUIREMENTS_UPLOAD_URL = "{}/requirements/upload".format(PROXY_API_PREFIX)
    STATUS_URL = "{}/proxy/health/status".format(PROXY_API_PREFIX)

def apply(request, proxy_url, source_notebook_path):
    request.log.debug("apply transformer({}) content to proxy({})...".format(source_notebook_path, proxy_url))

    if proxy_url:
        revise_proxy_api_url(request, proxy_url)

    paths = parse_notebook(request, source_notebook_path)

    if paths.get('requirements_path', None):
        files = {'file': open(paths.get('requirements_path'), 'rb')}
        request.log.debug("POST {} with files...".format(REQUIREMENTS_UPLOAD_URL))
        response = requests.post(REQUIREMENTS_UPLOAD_URL, files = files)
        request.log.debug(response.json())

    if paths.get('transformer_path', None):
        files = {'file': open(paths.get('transformer_path'), 'rb')}
        request.log.debug("POST {} with files...".format(TRANSFORMER_UPLOAD_URL))
        response = requests.post(TRANSFORMER_UPLOAD_URL, files = files)
        request.log.debug(response.json())

def reset(request, proxy_url, source_notebook_path):
    request.log.debug("reset transformer({}) content to proxy({})...".format(source_notebook_path, proxy_url))
    text = resources.read_text(res, 'transformer.ipynb', encoding = 'utf8', errors = 'strict')
    request.log.debug(text)
    nb_file = open(source_notebook_path, "w")
    nb_file.write(text)
    nb_file.close()

def info(request):
    request.log.debug("info")
    cwd = os.getcwd()
    proxy_url = ""
    nb_paths = []
    PROXY_URL_FILE_NAME = ".proxy"
    TRANSFORMER_NB_FILE_NAME = "transformer.ipynb"
    try:
        for root, dirs, files in os.walk(cwd):
            if PROXY_URL_FILE_NAME in files:
                path = os.path.join(root, PROXY_URL_FILE_NAME)
                proxy_url = open(path).readline().strip()
                break
    except Exception:
        pass

    request.log.debug("proxy_url: {}".format(proxy_url))
    try:
        for root, dirs, files in os.walk(cwd):
            if TRANSFORMER_NB_FILE_NAME in files:
                nb_paths.append(os.path.join(root, TRANSFORMER_NB_FILE_NAME))
    except Exception:
        pass
    request.log.debug("nb_paths: {}".format(nb_paths))

    UNAVAILABLE = "Unavailable"
    info = {
        "nb_paths": nb_paths,
        "proxy_url": proxy_url,
        "proxy_status": UNAVAILABLE
    }
    if proxy_url:
        revise_proxy_api_url(request, proxy_url)

        try:
            response = requests.get(STATUS_URL)
            if response.status_code == 200:
                info['proxy_status'] = response.json().get("status", UNAVAILABLE)
        except requests.exceptions.RequestException as e:
            request.log.warn(str(e))
    return info



import logging
import os

from modeldeploy_proxy_controller.common import logutils

LOG_DIR = os.getenv("HOME", ".")
LOG_BASENAME = "modeldeploy-proxy-controller.log"
LOG_FILE = os.path.join(LOG_DIR, LOG_BASENAME)

FMT_PREFIX = "%(asctime)s %(module)s:%(lineno)d [%(levelname)s] "
RPC_FMT_EXTRAS = "[TID=%(trans_id)s] [%(nb_path)s] "


def create_adapter(logger, trans_id = None, nb_path = None):
    """Create log Adapter."""
    extras = {
        "trans_id": trans_id or "",
        "nb_path": os.path.realpath(nb_path) if nb_path else ""
    }
    return logging.LoggerAdapter(logger, extras)


def setup_logging(request):
    """Configure logging."""
    # Set up root logger
    fmt = FMT_PREFIX + "%(message)s"
    _root_log = logutils.get_or_create_logger("", fmt = fmt, file_level = logging.INFO, log_path = LOG_FILE)
    _root_log.setLevel(logging.INFO)

    fmt = FMT_PREFIX + RPC_FMT_EXTRAS + "%(message)s"
    logutils.get_or_create_logger("modeldeploy_proxy_controller.rpc", fmt = fmt, log_path = LOG_FILE)

    # mute other loggers
    logging.getLogger('urllib3.connectionpool').setLevel(logging.CRITICAL)
    logging.getLogger('urllib3').setLevel(logging.INFO)



import os
import re
import warnings
from typing import Any, Dict
import nbformat as nb
from modeldeploy_proxy_controller.common import utils

TRANSFORMER_NB_METADATA_KEY = 'transformer'

REQUIREMENTS_TAG = 'requirements'
PREDICT_TAG = 'predict'
FUNCTIONS_TAG = 'functions'
CVAT_INFO_TAG = 'cvat-info'
CVAT_INVOKE_TAG = 'cvat-invoke'

SUPPORTED_TAGS = [
    REQUIREMENTS_TAG,
    PREDICT_TAG,
    FUNCTIONS_TAG,
    CVAT_INFO_TAG,
    CVAT_INVOKE_TAG
]

class NotebookProcessor():
    id = "nb"

    def __init__(self, nb_path: str, **kwargs):
        """Instantiate a new NotebookProcessor.

        Args:
            nb_path: Path to source notebook
        """
        self.nb_path = os.path.expanduser(nb_path)
        self.notebook = self._read_notebook()

    def _read_notebook(self):
        if not os.path.exists(self.nb_path):
            raise ValueError("NotebookProcessor could not find a notebook at path %s" % self.nb_path)
        return nb.read(self.nb_path, as_version=nb.NO_CONVERT)

    def parse_notebook(self):
        requirements_block = list()
        predict_block = list()
        functions_block = list()
        cvat_info_block = list()
        cvat_invoke_block = list()

        for c in self.notebook.cells:
            tag = self.get_cell_transformer_tag(c.metadata)
            if not tag:
                continue

            if REQUIREMENTS_TAG == tag:
                requirements_block.append(c.source);
                continue
            elif PREDICT_TAG == tag:
                predict_block.append(c.source)
                continue
            elif FUNCTIONS_TAG == tag:
                functions_block.append(c.source)
                continue
            elif CVAT_INFO_TAG == tag:
                cvat_info_block.append(c.source)
                continue
            elif CVAT_INVOKE_TAG == tag:
                cvat_invoke_block.append(c.source)
                continue
            else:
                continue
        return {
            REQUIREMENTS_TAG: requirements_block,
            PREDICT_TAG: predict_block,
            FUNCTIONS_TAG: functions_block,
            CVAT_INFO_TAG: cvat_info_block,
            CVAT_INVOKE_TAG: cvat_invoke_block
        }

    def get_cell_transformer_tag(self, metadata):
        """Parse a notebook's cell's metadata field.

        The UI widget writes the 'transformer' key-value in metadata filed as a 
        tag. Supported tags are defined by SUPPORTED_TAGS.

        Args:
            metadata (dict): a dict containing a notebook's cell's metadata

        Returns (str): parsed tag

        """
        if TRANSFORMER_NB_METADATA_KEY not in metadata or len(metadata[TRANSFORMER_NB_METADATA_KEY]) == 0:
            return None

        if not isinstance(metadata[TRANSFORMER_NB_METADATA_KEY], str):
            raise ValueError("Tag must be string. Found tag %s of type %s" % (metadata[TRANSFORMER_NB_METADATA_KEY], type(metadata[TRANSFORMER_NB_METADATA_KEY])))

        if metadata[TRANSFORMER_NB_METADATA_KEY] not in SUPPORTED_TAGS:
            raise ValueError("Unrecognized tag: {}".format(metadata[TRANSFORMER_NB_METADATA_KEY]))

        return metadata[TRANSFORMER_NB_METADATA_KEY]

    def write_transformer_python(self, source):
        python_file_path = os.path.dirname(self.nb_path) + '/transformer.py'
        python_file = open(python_file_path, "w+")
        python_file.write(source)
        return python_file_path

    def write_requirements_file(self, requirements):
        python_file_path = os.path.dirname(self.nb_path) + '/proxy_requirements.txt'
        python_file = open(python_file_path, "w+")
        python_file.write(requirements)
        return python_file_path



import os
import logging
from typing import List, Sequence

LOG_FORMAT = "%(asctime)s Model-Deployment-Controller {} %(levelname)s %(message)s"
DATE_FORMAT = "%Y-%m-%d %H:%M:%S"
BLANK_FORMAT = logging.Formatter(fmt="")

_loggers = dict()


class CustomLogRecord(logging.LogRecord):
    """Custom log record.

    We use a custom log record in order to create custom and compound log
    elements that can be used by the Formatter just like standard properties.
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.origin = f"{self.module}:{self.lineno}"
        # override the default levelname
        self.levelname = f"[{self.levelname}]"


def _suppress_handlers(log: logging.Logger) -> List[List[logging.Formatter]]:
    def _suppress_handlers_for_logger(log: logging.Logger) -> (
            List[logging.Formatter]):
        formatters = []
        for h in log.handlers:
            formatters.append(h.formatter)
            h.setFormatter(BLANK_FORMAT)
        return formatters

    formatters = [_suppress_handlers_for_logger(log)]
    while log.propagate and log.parent:
        log = log.parent
        formatters.append(_suppress_handlers_for_logger(log))
    return formatters


def _restore_handlers(log: logging.Logger,
                      formatters: Sequence[logging.Formatter]) -> None:
    for i in range(len(formatters)):
        for handler, fmt in zip(log.handlers, formatters[i]):
            handler.setFormatter(fmt)
        log = log.parent


class ControllerLogger(logging.Logger):

    def newline(self, lines: int = 1):
        original_formatters = _suppress_handlers(self)
        for _ in range(lines):
            self.info('')
        _restore_handlers(self, original_formatters)

logging.setLogRecordFactory(CustomLogRecord)
logging.setLoggerClass(ControllerLogger)

def _configure_handler(handler, level, formatter):
    handler.setLevel(level)
    handler.setFormatter(formatter)

def get_or_create_logger(module, name=None, level=logging.INFO, fmt=None, file_level=logging.DEBUG, file_fmt=None, log_path=None):
    """Get or create and return module's logger.

    Args:
        module: the module for which we request a logger
        name: the name/title of the logger if default format is used
        level: logging level for StreamHandlers
        fmt: override default format
        file_level: logging level for FileHandlers
        file_fmt: override default file format
        log_path: logfile path (if None, then no FileHandler is created)
    """
    global _loggers
    log = _loggers.get(module)
    if log:
        for h in log.handlers:
            # XXX: StreamHandler is FileHandler's parent class, so the order
            # XXX: should be like this
            if isinstance(h, logging.FileHandler):
                h.setLevel(file_level)
            elif isinstance(h, logging.StreamHandler):
                h.setLevel(level)
        return log

    # Set up logger
    log = logging.getLogger(module)
    log.propagate = False
    log.setLevel(logging.DEBUG)

    # Set up handlers
    log_fmt = fmt or LOG_FORMAT.format("%s" % name if name else "%(origin)s")
    stream_handler = logging.StreamHandler()
    _configure_handler(stream_handler, level, logging.Formatter(log_fmt, DATE_FORMAT))
    log.addHandler(stream_handler)

    if log_path:
        # if log_path is just a file name, dirname will be empty
        if os.path.dirname(log_path):
            os.makedirs(os.path.dirname(log_path), exist_ok=True)
        file_handler = logging.FileHandler(filename=os.path.abspath(log_path),
                                           mode='a')
        _configure_handler(file_handler, file_level,
                           logging.Formatter(file_fmt or log_fmt, DATE_FORMAT))
        log.addHandler(file_handler)

    _loggers[module] = log
    return log



from setuptools import setup

setup(
    name = 'modeldeploy-proxy-controller',
    version = '0.2.1',
    description = 'The JupyterLab backend component for modeldeploy-proxy.',
    author = 'ever cheng',
    author_email = 'ever_cheng@asus.com',
    license = 'Apache License Version 2.0',
    packages = [
        'modeldeploy_proxy_controller',
        'modeldeploy_proxy_controller.res',
        'modeldeploy_proxy_controller.common',
        'modeldeploy_proxy_controller.rpc',
        'modeldeploy_proxy_controller.processors'
    ],
    install_requires = [
        'IPython >= 7.6.0',
        'jupyter-client >=5.3.4, <7.0.0',
        'jupyter-core >= 4.6.0',
        'nbformat',
        'ipykernel >= 5.1.4',
        'notebook >= 6.0.0',
        'packaging > 20',
        'Flask >= 2.0.0'
    ],
    extras_require = {
        'dev': [
            'pytest',
            'pytest-clarity',
            'testfixtures',
            'pytest-cov',
        ]
    },
    entry_points = {
        'console_scripts': [
            'modeldeploy_proxy_controller=src.cli:main'
        ]
    },
    python_requires = '>=3.7.0',
    include_package_data = True,
    zip_safe = False
)



import os
import shutil
import logging
from modeldeploy_proxy_controller.processors.nbprocessor import NotebookProcessor, REQUIREMENTS_TAG, PREDICT_TAG, FUNCTIONS_TAG, CVAT_INFO_TAG, CVAT_INVOKE_TAG
from modeldeploy_proxy_controller.rpc.errors import RPCInternalError
import requests

def parse_notebook(request, source_notebook_path):
    request.log.debug("parse_notebook with path({})...".format(source_notebook_path))

    NEW_LINES = '\n\n';
    processor = NotebookProcessor(source_notebook_path)
    blocks = processor.parse_notebook()
    source = ''

    requirements = ''
    request.log.debug("Requirements({})...".format(len(blocks[REQUIREMENTS_TAG])))
    if len(blocks[REQUIREMENTS_TAG]) > 1:
        raise RuntimeError("Requirements tag must be at most 1!")
    elif len(blocks[REQUIREMENTS_TAG]):
        if requirements:
            requirements = '{}{}{}'.format(source, NEW_LINES, blocks[REQUIREMENTS_TAG][0])
        else:
            requirements = '{}'.format(blocks[REQUIREMENTS_TAG][0])

    if len(blocks[PREDICT_TAG]) > 1:
        raise RuntimeError("Predict tag must be at most 1!")
    elif len(blocks[PREDICT_TAG]):
        if source:
            source = '{}{}{}'.format(source, NEW_LINES, blocks[PREDICT_TAG][0])
        else:
            source = '{}'.format(blocks[PREDICT_TAG][0])

    if len(blocks[CVAT_INFO_TAG]) > 1:
        raise RuntimeError("CVAT info tag must be at most 1!")
    elif len(blocks[CVAT_INFO_TAG]):
        if source:
            source = '{}{}{}'.format(source, NEW_LINES, blocks[CVAT_INFO_TAG][0])
        else:
            source = '{}'.format(blocks[CVAT_INFO_TAG][0])

    if len(blocks[CVAT_INVOKE_TAG]) > 1:
        raise RuntimeError("CVAT invoke tag must be at most 1!")
    elif len(blocks[CVAT_INVOKE_TAG]):
        if source:
            source = '{}{}{}'.format(source, NEW_LINES, blocks[CVAT_INVOKE_TAG][0])
        else:
            source = '{}'.format(blocks[CVAT_INVOKE_TAG][0])

    if len(blocks[FUNCTIONS_TAG]):
        for function in blocks[FUNCTIONS_TAG]:
            if source:
                source = '{}{}{}'.format(source, NEW_LINES, function)
            else:
                source = '{}'.format(function)

    request.log.debug("Write contents...")
    request.log.debug(requirements)
    requirements_path = processor.write_requirements_file(requirements);
    transformer_path = processor.write_transformer_python(source);

    request.log.debug("requirements_path({})...".format(requirements_path))
    request.log.debug("transformer_path({})...".format(transformer_path))

    return {
        "requirements_path": requirements_path,
        "transformer_path": transformer_path
    }



import enum
from modeldeploy_proxy_controller.rpc.utils import serialize
from modeldeploy_proxy_controller.rpc.log import LOG_FILE

class Code(enum.Enum):
    """Error codes."""
    OK = 0
    IMPORT_ERROR = 1
    ENCODING_ERROR = 2
    NOT_FOUND = 3
    INTERNAL_ERROR = 4
    SERVICE_UNAVAILABLE = 5
    UNHANDLED_ERROR = 6


class _RPCError(Exception):
    """Generic RPC Error."""

    name = "rpcError"
    message = "RPC error"
    details = "You can find more information under %s" % LOG_FILE
    trans_id = -1

    def __init__(self, message=None, details=None, trans_id=None):
        if message:
            self.message = message
        if details:
            self.details = details
        if trans_id:
            self.trans_id = trans_id

    def to_dict(self):
        return {"code": self.code.value, "err_message": self.message,
                "err_details": self.details, "err_cls": self.name,
                "trans_id": self.trans_id}

    def serialize(self):
        return serialize(self.to_dict())


class RPCImportError(_RPCError):
    """Import Error."""
    name = "importError"
    code = Code.IMPORT_ERROR


class RPCEncodingError(_RPCError):
    """Encoding Error."""
    name = "encodingError"
    code = Code.ENCODING_ERROR


class RPCNotFoundError(_RPCError):
    """Not Found Error."""
    name = "notFoundError"
    code = Code.NOT_FOUND
    message = "Not Found"


class RPCInternalError(_RPCError):
    """Internal Error."""
    name = "internalError"
    code = Code.INTERNAL_ERROR
    message = "Internal Error"


class RPCServiceUnavailableError(_RPCError):
    """Service Unavailable Error."""
    name = "serviceUnavailableError"
    code = Code.SERVICE_UNAVAILABLE
    message = "Service is Unavailable"


class RPCUnhandledError(_RPCError):
    """Unhandled RPC Error."""

    name = "unhandledException"
    code = Code.UNHANDLED_ERROR
    message = "Unhandled exception during RPC execution"



import argparse
from argparse import RawTextHelpFormatter
from modeldeploy_proxy_controller.processors.nbprocessor import NotebookProcessor

ARGS_DESC = ""

def main():
    """Entry-point of CLI command."""
    parser = argparse.ArgumentParser(description = ARGS_DESC, formatter_class = RawTextHelpFormatter)
    general_group = parser.add_argument_group('General')
    general_group.add_argument('--nb', type=str, help='Path to source JupyterNotebook.', required=True)
    general_group.add_argument('--debug', action='store_true')

    args = parser.parse_args()

    processor = NotebookProcessor(args.nb, {})
    processor.parse_notebook()

if __name__ == "__main__":
    main()
